@@ -269,12 +269,7 @@ def get_gtf(chrom, build="19",source="ensembl"):
         
 ####################################################################################################################   
         
-def gwaslab_info():
-    dic={
-   "version":"3.3.18",
-   "release_date":"20230108"
-    }
-    return dic       
+    
         
         
         
