@@ -24,6 +24,7 @@ from mpl_toolkits.axes_grid1.inset_locator import inset_axes
 from mpl_toolkits.axes_grid1.inset_locator import mark_inset
 import gc as garbage_collect
 from adjustText import adjust_text
+from gwaslab.textreposition import adjust_text_position
 
 # 20220310 ######################################################################################################
 
@@ -66,7 +67,6 @@ def mqqplot(insumstats,
           tabix=None,
           mqqratio=3,
           bwindowsizekb = 100,
-          large_number = 10000000000,
           density_color=False,
           density_range=(0,15),
           density_trange=(0,10),
@@ -79,8 +79,11 @@ def mqqplot(insumstats,
           anno_alias={},
           anno_d={},
           anno_args={},
+          anno_style="right",
           anno_fixed_arm_length=None,
           anno_source = "ensembl",
+          anno_adjust=False,
+          anno_max_iter=100,
           arm_offset=50,
           arm_scale=1,
           arm_scale_d=None,
@@ -320,6 +323,14 @@ def mqqplot(insumstats,
 # Density #####################################################################################################              
     if "b" in mode and "DENSITY" not in sumstats.columns:
         if verbose:log.write(" -Calculating DENSITY with windowsize of ",bwindowsizekb ," kb")
+        
+        large_number = 1000000000
+        for i in range(7):
+            if sumstats[pos].max()*10 >  large_number:
+                large_number = int(large_number * 10)
+            else:
+                break
+        
         stack=[]
         sumstats["TCHR+POS"] = sumstats[chrom]*large_number +  sumstats[pos]
         sumstats = sumstats.sort_values(by="TCHR+POS")
@@ -825,51 +836,73 @@ def mqqplot(insumstats,
         elif mtitle:
             ax1.set_title(mtitle,fontsize=title_fontsize,family="sans-serif")
             
-       
+        # add annotation arrows and texts
         if anno and (to_annotate.empty is not True):
             #initiate a list for text and a starting position
             text = []
             last_pos=0
             anno_count=0
-            
+            to_annotate = to_annotate.sort_values(by=[chrom,pos])
             ## log : annotation column
             if anno==True:
                     annotation_col="CHR:POS"
             elif anno:
                     annotation_col=anno
             if verbose: log.write(" -Annotating using column "+annotation_col+"...")
+                            ## calculate y span
+            if region is not None:
+                y_span = region[2] - region[1]
+            else:
+                y_span = sumstats["i"].max()-sumstats["i"].min()
             
-            ##   
+            if verbose: log.write(" -Adjusting text positions with repel_force={}...".format(repel_force))
+            if anno_style == "expand" :
+                to_annotate.loc[:, "ADJUSTED_i"] = adjust_text_position(to_annotate["i"].values.copy(), y_span, repel_force,max_iter=anno_max_iter,log=log,verbose=verbose)
+            ##  iterate through variants to be annotated
+            anno_to_adjust_list = list()
             for rowi,row in to_annotate.iterrows():
+                
                 # avoid text overlapping
-                ## calculate y span
-                if region is not None:
-                    y_span = region[2] - region[1]
+                ## adjust x to avoid overlapping################################################################
+                if anno_style == "right" :
+                    #right style
+                    if row["i"]>last_pos+repel_force*y_span:
+                        last_pos=row["i"]
+                    else:
+                        last_pos+=repel_force*y_span
+                elif anno_style == "expand" :
+                    #expand style
+                    last_pos = row["ADJUSTED_i"]
+                    anno_args["rotation"] = 90
+                elif anno_style == "tight" :
+                    #tight style
+                    anno_fixed_arm_length = 1
+                    anno_adjust = True
+                    anno_args["rotation"] = 90
                 else:
-                    y_span = sumstats["i"].max()-sumstats["i"].min()
-                
-                ## adjust x to avoid overlapping
-                if row["i"]>last_pos+repel_force*y_span:
-                    last_pos=row["i"]
-                else:
-                    last_pos+=repel_force*y_span
-                
+                    pass
+                ################################################################
+                #shrink or increase the arm
                 if arm_scale_d is not None:
                     if anno_count not in arm_scale_d.keys():
                         arm_scale =1
                     else:
                         arm_scale = arm_scale_d[anno_count]
-                
+                ################################################################
+
                 # vertical arm length in pixels
                 armB_length_in_point = ax1.transData.transform((skip,1.15*maxy))[1]-ax1.transData.transform((skip, row["scaled_P"]+1))[1]-arm_offset/2
-                #
+                # scale if needed
                 armB_length_in_point = armB_length_in_point*arm_scale
+                ################################################################
                 if arm_scale>=1:
                     armB_length_in_point= armB_length_in_point if armB_length_in_point>0 else ax1.transData.transform((skip, maxy+2))[1]-plot.transData.transform((skip,  row["scaled_P"]+1))[1] 
-                
+                ###if anno_fixed_arm_length #############################################################
                 if anno_fixed_arm_length is not None:
                     anno_fixed_arm_length_factor = ax1.transData.transform((skip,anno_fixed_arm_length))[1]-ax1.transData.transform((skip,0))[1] 
                     armB_length_in_point = anno_fixed_arm_length_factor
+                ################################################################################################################################
+                # annotation alias
                 if anno==True:
                     if row[snpid] in anno_alias.keys():
                         annotation_text = anno_alias[row[snpid]]
@@ -888,13 +921,18 @@ def mqqplot(insumstats,
                 
                 xy=(row["i"],row["scaled_P"]+0.2)
                 xytext=(last_pos,1.15*maxy*arm_scale)
+                
                 if anno_fixed_arm_length is not None:
                     armB_length_in_point = anno_fixed_arm_length
                     xytext=(row["i"],row["scaled_P"]+0.2+anno_fixed_arm_length)
+                
                 if anno_count not in anno_d.keys():
+                    #arrowargs = dict(arrowstyle="-|>",relpos=(0,0),color="#ebebeb",
+                    #                         connectionstyle="arc,angleA=0,armA=0,angleB=90,armB="+str(armB_length_in_point)+",rad=0")
                     arrowargs = dict(arrowstyle="-|>",relpos=(0,0),color="#ebebeb",
                                              connectionstyle="arc,angleA=0,armA=0,angleB=90,armB="+str(armB_length_in_point)+",rad=0")
                 else:
+                    # adjuest direction
                     xy=(row["i"],row["scaled_P"])
                     if anno_d[anno_count] in ["right","left","l","r"]:
                         if anno_d[anno_count]=="right" or anno_d[anno_count]=="r": 
@@ -929,8 +967,10 @@ def mqqplot(insumstats,
                 anno_default = {"rotation":40,"style":"italic","ha":"left","va":"bottom","fontsize":anno_fontsize,"fontweight":fontweight}
                 for key,value in anno_args.items():
                     anno_default[key]=value
-                   
-                ax1.annotate(annotation_text,
+
+                if anno_adjust==True:
+                    arrowargs=dict(arrowstyle='-|>', color='grey', shrinkA=10, linewidth=0.1, relpos=(0,0.5))
+                anno_to_adjust = ax1.annotate(annotation_text,
                          xy=xy,
                          xytext=xytext,
                          bbox=bbox_para,
@@ -938,12 +978,31 @@ def mqqplot(insumstats,
                          zorder=100,
                          **anno_default
                          )
-                 
+                anno_to_adjust_list.append(anno_to_adjust) 
                 anno_count +=1
-
+            #anno_adjust_keyargs = {"arrowprops":dict(arrowstyle='->', color='grey', linewidth=0.1,relpos=(0.5,0.5))}
+            if anno_adjust==True:
+                if verbose: log.write(" -Auto-adjusting text positions...")
+                adjust_text(texts = anno_to_adjust_list,
+                            autoalign=False, 
+                            only_move={'points':'x', 'text':'x', 'objects':'x'},
+                            ax=ax1,
+                            precision=0.02,
+                            force_text=(repel_force,repel_force),
+                            expand_text=(1,1),
+                            expand_objects=(0,0),
+                            expand_points=(0,0),
+                            va="bottom",
+                            ha='left',
+                            avoid_points=False,
+                            lim =100
+                            #kwargs = anno_adjust_keyargs
+                            )
             
         else:
             if verbose: log.write(" -Skip annotating")
+        
+        # gene track (ax3) text adjustment
         if (gtf_path is not None ) and ("r" in mode):
             if len(texts_to_adjust_middle)>0:
                 adjust_text(texts_to_adjust_middle,
@@ -1259,3 +1318,4 @@ def closest_gene(x,data,chrom="CHR",pos="POS",maxiter=20000,step=50):
             return distance,"intergenic"
         else:
             return 0,",".join(gene)
+
